#! /bin/jsh -
### ====================================================================
### Search the Library of Congress catalog via its Z39.50 interface for
### one or more publications by author, title, ISBN, or ISSN, and
### convert the US MARC records found to BibTeX on stdout.  It is NOT
### necessary to remove hyphens from ISBN or ISSN data.
###
### Usage:
###	cattobib [ --? ] \
###		 [ --byxxx ] \
###		 [ --help ] \
###		 [ --keep-files ] \
###		 [ --logfile new-filename ] \
###		 [ --quiet ] \
###		 [ --server server-address ] \
###		 [ --test ] \
###		 [ --version ] \
###		 search-key-1 search-key-2 search-key-3 ... >BibTeX-file
###
### Options may be abbreviated to any unique leading prefix, and may
### begin with either one or two hyphens.  Any option beginning -by...
### is assumed to be a bibsort option, and is passed verbatim to it.
###
### [15-Nov-2004]
### ====================================================================

IFS='
 	'
umask 077


### Root of local installation tree
prefix=/usr/local
bindir=$prefix/bin

TMPDIR=${TMPDIR-/tmp}

### Paths and directories:
PROGRAM=cattobib
VERSION=0.00
SHRLIBDIR=${TESTSHRLIBDIR-$prefix/share/$PROGRAM/${PROGRAM}-$VERSION}

### Programs needed:
AWK=/usr/local/bin/mawk
BIBCLEAN=/usr/local/bin/bibclean
BIBLABEL=/usr/local/bin/biblabel
BIBORDER=/usr/local/bin/biborder
BIBSORT=/usr/local/bin/bibsort
CAT=/usr/local/bin/cat
CITESUB=/usr/local/bin/citesub
DATE=/usr/local/bin/date
EXPECT=/usr/local/bin/expect
HOSTNAME=`/usr/local/bin/hostname`
LOGGER=/usr/local/bin/tee
TEE=/usr/local/bin/tee
YAZ_CLIENT=/usr/local/bin/yaz-client
Z39_50_TO_BIBTEX=$SHRLIBDIR/usmarc-to-bibtex.awk

### Files, flags and variables:
BIBCLEANFLAGS="-max-width 0"
BIBCLEANFLAGS=
BIBSORTFLAGS=
DEFAULT_Z39_50_SERVER="z3950.loc.gov:7090/Voyager"
FIFO=$TMPDIR/$PROGRAM.fifo.$$
FORMAT="USMARC"
LABELFILE=$TMPDIR/$PROGRAM.label.$$
LOGFILE=$TMPDIR/$PROGRAM.$HOSTNAME.`$DATE +%Y.%m.%d.%H.%M.%S`
LOGFILE=/dev/null
PACKAGE_DATE="19-Nov-2004"
PACKAGE_VERSION="0.00"
TEST_MODE=no
TMPBIBTEXFILE=$TMPDIR/$PROGRAM.bib.$$

# Z39.50 catalog servers:
Z39_50_SERVER="$DEFAULT_Z39_50_SERVER"

# Ensure consistent sorting across all platforms
LC_ALL=C
export LC_ALL

trap "rm -f $FIFO $LABELFILE $TMPBIBTEXFILE"  EXIT HUP INT QUIT PIPE TERM

error_and_exit()
{
    echo "ERROR: ""$@" 1>&2
    exit 1
}

usage()
{
    cat <<EOF
$0 [--?]
		    [ --byxxx ]
		    [ --help ]
		    [ --keep-files ]
		    [ --logfile new-filename ]
		    [ --quiet ]
		    [ --server server-address ]
		    [ --test ]
		    [ --version ]
		    search-key-1 search-key-2 search-key-3 ... >BibTeX-file

Search keys should be either ISBNs, or quoted title strings.  All
searches are completed before the data is processed further to produce
BibTeX entries, so if you interrupt execution, there will likely be no
useful output.

The server can be supplied as an explicit Z39.50 address, such as

			 $DEFAULT_Z39_50_SERVER

(the default server at the US Library of Congress), or more usually, by
a shorthand name (vertical bars separate alternatives, and asterisk
matches any word with that prefix):

	alberta | ab			University of Alberta
	amherst | umass			University of Massachusetts, Amherst
	boulder | co			University of Colorado, Boulder
	british | br			British Library
	calgary				University of Calgary
	congress | lc			US Library of Congress
	copac | uk			COPAC (union of 24 research-university 
					catalogs in the UK and Ireland)
	denmark | dk			Royal Library of Denmark
	dsb | dsl			Danish State Library
	duke				Duke University
	florida | fl			Florida Center for Library Automation
	marriott | ut			University of Utah Marriott Library
	melbourne			University of Melbourne
	melvyl | ca			University of California MELVYL catalog
	minn* | mn			University of Minnesota
	newyork | ny			New York University
	nla | au			National Library of Australia
	nlnz | nz			National Library of New Zealand
	norway | no			National Library of Norway
	nsw | unsw			University of New South Wales
	odense | sdu			University of Southern Denmark
	oxford | ox*			Oxford University
	sweden | se			National Library of Sweden
	texas | tx			University of Texas at Austin
	toronto				University of Toronto
	usc				University of Southern California

Please note that data from ANY of these libraries are likely to be error
ridden: it is best to repeat your searches with three or more servers in
order to improve data quality and reliability.

EOF
}

usage_and_exit()
{
    usage
    exit 0
}

version_and_exit()
{
    echo "This is `basename $0` version $PACKAGE_VERSION [$PACKAGE_DATE]"
    exit 0
}

while test $# -gt 0
do
	case $1 in

	'--?' | '-?' )
		usage_and_exit
		;;

	--by* | -by*)
		shift
		BIBSORT=$bindir/bibsort
		BIBSORTFLAGS="$BIBSORTFLAGS $1"
		;;

	--help | --hel | --he | --h | \
	-help | -hel | -he | -h )
		usage_and_exit
		;;

	--keep-files | --keep-file | --keep-fil | --keep-fi | --keep-f | --keep- | \
	--keep | --kee | --ke | --k | \
	-keep-files | -keep-file | -keep-fil | -keep-fi | -keep-f | -keep- | \
	-keep | -kee | -ke | -k )
		trap '' EXIT HUP INT QUIT PIPE TERM
		;;

	--log | --lo | --l | \
	-log | -lo | -l )
		shift
		test -f "$LOGFILE" && error_and_exit "Log file $LOGFILE already exists"
		;;

	--quiet | --quie | --qui | --qu | --q | \
	-quiet | -quie | -qui | -qu | -q )
		LOGGER=$CAT
		LOGFILE=
		;;

	--server | --serve | --serv | --ser | --se | --s | \
	-server | -serve | -serv | -ser | -se | -s )
		shift
		## See http://www.indexdata.dk/targettest/u.php for a long list
		## See http://www.bs.dk/danzig for pointers to a list of Danish servers
		## See http://www.nla.gov.au/apps/libraries?action=ListTargets for Australian servers

		case $1 in

		### These work
		alberta | ab )				Z39_50_SERVER="dra.library.ualberta.ca:210/Marion" ;;
		amherst | umass )			Z39_50_SERVER="umlibr.library.umass.edu:210/INNOPAC" ;;
		boulder | co )				Z39_50_SERVER="libraries.colorado.edu:210/INNOPAC" ;;
		british | br )				Z39_50_SERVER="z3950cat.bl.uk:9909/BLAC"
							Z39_50_TO_BIBTEX="$SHRLIBDIR/sutrs-to-bibtex.awk"
							FORMAT="SUTRS"
							URL="http://www.bl.uk/catalogues/z3950copacaccess.html"
							;;
		calgary )				Z39_50_SERVER="clavis.ucalgary.ca:2200/UNICORN" ;;
		congress | lc )				Z39_50_SERVER="z3950.loc.gov:7090/Voyager" ;;
		copac | uk )				Z39_50_SERVER="z3950.copac.ac.uk:2020" ;;
		denmark | dk )				Z39_50_SERVER="z3950.kb.dk:2100/KGL01" ;;
		dsb | dsl )				Z39_50_SERVER="zeus.statsbiblioteket.dk:210/Horizon" ;;
		duke )					Z39_50_SERVER="ducatalog.lib.duke.edu:210/MARION" ;;
		florida | fl )				Z39_50_SERVER="z3950.fcla.edu:210/AM" ;;
		marriott | ut )				Z39_50_SERVER="cos.lib.utah.edu:6060" ;;
		melbourne )				Z39_50_SERVER="library.unimelb.edu.au:210/INNOPAC" ;;
		melvyl | ca )				Z39_50_SERVER="melvyl.cdlib.org:210/CDL90" ;;
		minn* | mn )				Z39_50_SERVER="umnlib.oit.umn.edu:9909/MNCAT" ;;
		newyork | ny )				Z39_50_SERVER="bobcat.nyu.edu:210/ADVANCE" ;;
		nla | au )				Z39_50_SERVER="catalogue.nla.gov.au:7090/Voyager" ;;
		nlnz | nz )				Z39_50_SERVER="zed.natlib.govt.nz:210/pinz" ;;
		norway | no )				Z39_50_SERVER="z3950.nb.no:2100/Norper" ;;
		nsw | unsw )				Z39_50_SERVER="aleph.nun.unsw.edu.au:80/NUN" ;;
		odense | sdu )				Z39_50_SERVER="odin2.bib.sdu.dk:210/Horizon" ;;
		oxford | ox* )				Z39_50_SERVER="library.ox.ac.uk:210/ADVANCE" ;;
		sweden | se )				Z39_50_SERVER="z3950.libris.kb.se:210/libr" ;;
		texas | tx )				Z39_50_SERVER="db.lib.utexas.edu:210/utcat" ;;
		toronto )				Z39_50_SERVER="sirsi.library.utoronto.ca:2200/UNICORN" ;;
		usc )					Z39_50_SERVER=library.usc.edu:2200/unicorn ;;

		### These do not work
		## bell* | z3950.bell-labs.com )	Z39_50_SERVER=z3950.bell-labs.com/books ;;
		## inquirion* | teratext* )		Z39_50_SERVER="z3950.inquirion.com:210" ;;
		## melvyl* | calif* )			Z39_50_SERVER=melvyl.ucop.edu/CAT ;;

		### Default: user-provided address
		*)					Z39_50_SERVER="$1" ;;
		esac
		;;

	--test | --tes | --te | --t | \
	-test | -tes | -te | -t )
		TEST_MODE=yes
		;;

	--version | --versio | --versi | --vers | --ver | --ve | --v | \
	-version | -versio | -versi | -vers | -ver | -ve | -v )
		version_and_exit
		;;

	-*)
		error_and_exit unrecognized option "$1"
		;;

	*)
		break
		;;
	esac
	shift
done

test $LOGGER = $CAT && LOGFILE=

# mkfifo $FIFO
for f in "$@"
do
	case $f in
	[0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9Xx] | \
	[-0-9][-0-9][-0-9][-0-9][-0-9][-0-9][-0-9][-0-9][-0-9][-0-9][-0-9][-0-9][-0-9Xx])
		# Remove hyphens from ISBNs
		f=`echo $f | sed -e s/-//g`
		flags="@attr 1=7"
		;;
	*)
		flags=""
		;;
	esac
	test $LOGGER = $CAT || echo "%% Searching [$Z39_50_SERVER] for [$f]: flags = [$flags]" 1>&2

if test "$TEST_MODE" = yes
then			# test input from stdin instead of from Z39.50 server
	$CAT >$FIFO 
else
	$EXPECT >$FIFO <<EOFILE
		spawn $YAZ_CLIENT
		expect "Z>"
		send -- "open $Z39_50_SERVER\r"
		expect "Z>"
		send -- "format $FORMAT\r"
		expect "Z>"
		send -- "find $flags \"$f\"\r"
		expect "Number of hits: "
		expect "Z>"
		# For now, just try to display first five records
		send -- "show\r"
		expect "Z>"
		send -- "show\r"
		expect "Z>"
		send -- "show\r"
		expect "Z>"
		send -- "show\r"
		expect "Z>"
		send -- "show\r"
		send -- "close\r"
EOFILE
fi

	cat $FIFO |
		$AWK -f $Z39_50_TO_BIBTEX
done > $TMPBIBTEXFILE

$BIBLABEL $TMPBIBTEXFILE >$LABELFILE

$CITESUB -f $LABELFILE $TMPBIBTEXFILE 2>/dev/null |
	 $BIBSORT $BIBSORTFLAGS |
		 $BIBORDER |
			 $BIBCLEAN $BIBCLEANFLAGS 2>/dev/null |
				$AWK -f $SHRLIBDIR/iso8859-1-to-TeX.awk |
					$LOGGER $LOGFILE
